Package: ClustIRR
Type: Package
Title: Clustering of Immune Receptor Repertoires
Version: 1.9.35
Authors@R: c(
    person("Simo", "Kitanovski", email = "simokitanovski@gmail.com", 
    role = c("aut", "cre"), comment=c(ORCID="0000-0003-2909-5376")),
    person("Kai", "Wollek", email = "kai.wollek@uni-duisburg-essen.de", 
    role = c("aut"), comment=c(ORCID="0009-0008-5941-9160")))
Description: ClustIRR analyzes repertoires of B- and T-cell receptors. It 
    starts by identifying communities of immune receptors with similar 
    specificities, based on the sequences of their complementarity-determining 
    regions (CDRs). Next, it employs a Bayesian probabilistic models to 
    quantify differential community occupancy (DCO) between repertoires, 
    allowing the identification of expanding or contracting communities 
    in response to e.g. infection or cancer treatment.
License: GPL-3 + file LICENSE
Depends: R (>= 4.3.0)
Imports: grDevices, igraph, methods, Rcpp (>= 0.12.0), RcppParallel (>=
        5.0.1), reshape2, rstan (>= 2.18.1), rstantools (>= 2.4.0),
        stats, stringdist, utils, posterior, visNetwork, dplyr, tidyr,
        ggplot2, ggforce, scales, msa, Biostrings, RADanalysis,
        ggseqlogo, rBLAST
Suggests: BiocStyle, knitr, testthat, ggrepel, patchwork, htmlwidgets
SystemRequirements: GNU make, ncbi-blast+
Encoding: UTF-8
LazyData: false
NeedsCompilation: yes
biocViews: Clustering, ImmunoOncology, SingleCell, Software,
        Classification, Bayesian, BiomedicalInformatics,
        ImmunoOncology, MathematicalBiology
RoxygenNote: 7.2.3
VignetteBuilder: knitr
URL: https://github.com/snaketron/ClustIRR
BugReports: https://github.com/snaketron/ClustIRR/issues
Biarch: true
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
git_url: https://git.bioconductor.org/packages/ClustIRR
git_branch: devel
git_last_commit: ababdf1
git_last_commit_date: 2026-04-21
Repository: Bioconductor 3.23
Date/Publication: 2026-04-22
Packaged: 2026-04-22 21:07:48 UTC; biocbuild
Author: Simo Kitanovski [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2909-5376>),
  Kai Wollek [aut] (ORCID: <https://orcid.org/0009-0008-5941-9160>)
Maintainer: Simo Kitanovski <simokitanovski@gmail.com>
